#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_UPGRADE_RESTRICTED="page_upgrade_restricted"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGES_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB"
PAGES_UPGRADE="$PAGE_UPGRADE_RESTRICTED $PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB"
PAGES_UNINST="$PAGE_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

IsUpgradable()
{
	true
}

NeedMigrateMDB5()
{
	local prev_version="$1"
	if [ -z "$prev_version" ] || [ "$prev_version" -le "$MIGRATE_DB_VERSION" ]; then
		true
		return
	fi
	false
	return
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

GetPreviousPkgBuildNumber()
{
	local old_pkg_ver="$SYNOPKG_OLD_PKGVER"
	if [ -z "${old_pkg_ver}" ]; then
		# SYNOPKG_OLD_PKGVER does not exist in un-install stage of upgrade
		old_pkg_ver="$SYNOPKG_PKGVER"
	fi

	local old_build_number=$(echo "$old_pkg_ver" | cut -d- -f2)
	echo "$old_build_number"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

#!/bin/bash

PKG_NAME="vtigerCRM"
PKG_DIR="vtigercrm"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
CONF_FILE="config.inc.php"
DB_SETTING="modules/Install/models/Utils.php"
MIGRATE_DB_VERSION="0116"
INIT_DB_NAME="vtigercrm"
INIT_DB_USER="vtigercrm_user"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"
MANUAL_DOWNLOAD_URL="https://sy.to/ablgm"

GetCurrentVersion()
{
	local ver_path="$WEBSITE_ROOT/vtigerversion.php"
	local cur_version=""
	if [ -s "$ver_path" ]; then
		cur_version=$(php -r "include('$ver_path'); echo \$vtiger_current_version;")
		echo "${cur_version}"
	fi
}

IsUpgradable()
{
	local current_version=$(GetCurrentVersion)
	local allow_versions="7.3 7.3.0 7.4.0"
	for allow_version in $allow_versions; do
		if [ "$current_version" == "$allow_version" ]; then
			true
			return
		fi
	done
	false
}

IsIncConfigEmpty()
{
	local prev_pkg_path="$1"
	local website_inc_config="${prev_pkg_path}/config.inc.php"
	[ ! -s "$website_inc_config" ];
}

CustomHasRunWebsiteSetup()
{
	local prev_pkg_path="$1"
	# this behavior refers to isInstalled() in vtigercrm/includes/main/WebUI.php
	if IsIncConfigEmpty $prev_pkg_path ; then
		false
		return
	fi
	true
}

CustomGetBackupPkgPath()
{
	local backup_prefix_path=$(CustomGetBackupPrefix)
	local backup_pkg_path=""
	local pkg_dir_540="vtigercrm-5.4.0"

	backup_pkg_path="$backup_prefix_path/$PKG_DIR"
	if [ ! -d "$backup_pkg_path" ]; then
		backup_pkg_path="$backup_prefix_path/$pkg_dir_540"
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ]; then
		if [ -s "$path/$CONF_FILE" ]; then
			info_output=$(grep "dbconfig\['$grep_info'\]" "$path/$CONF_FILE" | cut -d"'" -f4)
		elif [ -s "$path/$DB_SETTING" ]; then
			info_output=$(grep "$grep_info' =>" "$path/$DB_SETTING" | cut -d"'" -f4)
		fi
	fi
	echo "$info_output"

}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "db_name"
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "db_username"
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "db_password"
}
wizard_db_settings="设置 vtigerCRM 数据库"
wizard_db_name_desc="请为 vtigerCRM 数据库输入数据库名称。 "
wizard_db_name_label="数据库名称"
wizard_m10_desc="请输入 MariaDB 10 管理员的帐户密码，继续 vtigerCRM 数据库设置。"
wizard_admin_acc="帐号"
wizard_admin_pass="密码"
wizard_set_db_desc="若要使用现有/全新数据，请为 vtigerCRM 创建数据库专属帐户。"
wizard_set_data_title="设置 vtigerCRM"
wizard_found_backup="套件保留数据不兼容"
wizard_warn_data_backup_deprecated="此版本与您系统中的此套件保留数据不兼容。<br>如果安装此版本，保留数据将被删除。<br>如果要留存保留数据，请安装与上一次安装相同版本的套件。（以前版本：{prev_version}）"
wizard_db_user_account_desc="数据库用户帐户"
wizard_db_user_password_desc="数据库用户密码"
wizard_migrate_note="最新版本的 vtigerCRM 仅支持 MariaDB 10。<br>系统已检测到现有 MariaDB 5 数据库。<br>在下一步中，系统会将 MariaDB 5 数据库迁移至 MariaDB 10。<br><br>请输入 MariaDB 5 管理员的帐户密码，以继续安装。"
wizard_migrate_title="迁移 vtigerCRM 数据库"
wizard_remove_msql_title="删除 vtigerCRM 数据库"
wizard_remove_msql_desc="如果移除 vtigerCRM 数据库，所有数据将会删除。"
wizard_msql_password_desc_remove="请输入数据库管理员的用户凭据以删除 vtigerCRM 数据库。"
wizard_admin_acc="帐号"
wizard_admin_pass="密码"
wizard_unable_to_upgrade_title="无法更新到此版本"
wizard_upgrade_from_prev_release_required_desc="若要将套件更新到此版本，请先更新到<a href=\"{1}\" target=\"_blank\">前一版本</a>。"
PageUninst()
{
cat << EOF
{
	"step_title": "$wizard_remove_msql_title",
	"items": [{
		"type": "multiselect",
		"subitems": [{
			"key": "pkgwizard_remove_mysql",
			"defaultValue": true,
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"desc": "$wizard_msql_password_desc_remove",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root"
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
