#!/bin/bash

# define main function
MAIN_INSTALL="main_install"
MAIN_UPGRADE="main_upgrade"
MAIN_UNINST="main_uninst"
# define each page's name
PAGE_MARIADB5="page_mariadb5"
PAGE_MARIADB10="page_mariadb10"
PAGE_RESTORE="page_restore"
PAGE_UPGRADE_RESTRICTED="page_upgrade_restricted"
PAGE_DB="page_db"
PAGE_UNINST="page_uninst"
DEFAULT="enu"
# set out file name
OUTPUT_INSTALL="install_uifile"
OUTPUT_UPGRADE="upgrade_uifile"
OUTPUT_UNINST="uninstall_uifile"
INIT_DB_NAME="" # need to be overwrited
INIT_DB_USER="" # need to be overwrited
INFO_FILE="" # need to be overwrited
BACKUP_PATH="" # need to be overwrited
# here to define what page is necessary for each stage: install, upgrade, and uninstall
PAGES_INST="$PAGE_MARIADB5 $PAGE_RESTORE $PAGE_MARIADB10 $PAGE_DB"
PAGES_UPGRADE="$PAGE_UPGRADE_RESTRICTED $PAGE_MARIADB5 $PAGE_MARIADB10 $PAGE_DB"
PAGES_UNINST="$PAGE_UNINST"
MIGRATE_DB_VERSION="-1"

MainCreate()
{
	local output="$1"
	local pages="$2"
	local main_function="$3"
	local lan
	for dir in lang/*; do
		lan=$(basename "$dir")
		cat "wizard_common" "wizard_customized" > "${output}_${lan}.sh"
		CustomWizCreate "${output}_${lan}.sh"
		cat "lang/$lan" $pages "$main_function" >> "${output}_${lan}.sh"
	done
	chmod +x "${output}"*.sh
	cp -a "${output}_${DEFAULT}.sh" "${output}.sh"
}

IsUpgradable()
{
	true
}

NeedMigrateMDB5()
{
	local prev_version="$1"
	if [ -z "$prev_version" ] || [ "$prev_version" -le "$MIGRATE_DB_VERSION" ]; then
		true
		return
	fi
	false
	return
}

CustomHasRunWebsiteSetup()
{
	local path="$1"
	[ -s "$path/$CONF_FILE" ]
}

CustomWizCreate()
{
	:
}

CustomSetBackupPath()
{
	BACKUP_PATH="$(CustomGetBackupPkgPath)"
}

CustomMainInstall()
{
	:
}

CustomGetValueFromMetaFile()
{
	local key="$1"
	local backup_info="/var/packages/.$PKG_DIR.conf"
	local syno_conf="/var/packages/$PKG_NAME/synology.conf"
	local record_file=""

	if [ -s "$INFO_FILE" ]; then
		record_file="$INFO_FILE"
	elif [ -s "$backup_info" ]; then
		record_file="$backup_info"
	elif [ -s "$syno_conf" ]; then
		record_file="$syno_conf"
	fi
	get_key_value "$record_file" "$key"
}

GetPreviousPkgBuildNumber()
{
	local old_pkg_ver="$SYNOPKG_OLD_PKGVER"
	if [ -z "${old_pkg_ver}" ]; then
		# SYNOPKG_OLD_PKGVER does not exist in un-install stage of upgrade
		old_pkg_ver="$SYNOPKG_PKGVER"
	fi

	local old_build_number=$(echo "$old_pkg_ver" | cut -d- -f2)
	echo "$old_build_number"
}

CustomGetBackupPrefix()
{
	local backup_prefix="/$(readlink /var/services/web | cut -d/ -f2)/@appstore/.$PKG_DIR"
	echo "$backup_prefix"
}

CustomGetBackupPkgPath()
{
	local backup_pkg_path="$(CustomGetBackupPrefix)/$PKG_DIR"
	echo "$backup_pkg_path"
}

# we define pages in fuction not in global variables,
# because functions in wizard steps will be merged after calling MainCreate.
CustomSetPages()
{
	NEW_INSTALL_PAGES="$(PageM10),$(PageDB)"
	RESTORE_PAGES="$(PageRestore),$(PageM10),$(PageDB)"
	MIGRATE_PAGES="$(PageM5),$(PageRestore),$(PageM10),$(PageDB)"
}

CustomParseDBConf()
{
	:
}

CustomGetDBName()
{
	:
}

CustomGetDBUser()
{
	:
}

CustomGetDBPass()
{
	:
}

CustomPageActivate()
{
	:
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

#!/bin/bash

PKG_NAME="vtigerCRM"
PKG_DIR="vtigercrm"
WEBSITE_ROOT="/var/services/web/$PKG_DIR"
CONF_FILE="config.inc.php"
DB_SETTING="modules/Install/models/Utils.php"
MIGRATE_DB_VERSION="0116"
INIT_DB_NAME="vtigercrm"
INIT_DB_USER="vtigercrm_user"
INFO_FILE="/usr/syno/etc/packages/$PKG_NAME/$PKG_DIR.conf"
MANUAL_DOWNLOAD_URL="https://sy.to/ablgm"

GetCurrentVersion()
{
	local ver_path="$WEBSITE_ROOT/vtigerversion.php"
	local cur_version=""
	if [ -s "$ver_path" ]; then
		cur_version=$(php -r "include('$ver_path'); echo \$vtiger_current_version;")
		echo "${cur_version}"
	fi
}

IsUpgradable()
{
	local current_version=$(GetCurrentVersion)
	local allow_versions="7.3 7.3.0 7.4.0"
	for allow_version in $allow_versions; do
		if [ "$current_version" == "$allow_version" ]; then
			true
			return
		fi
	done
	false
}

IsIncConfigEmpty()
{
	local prev_pkg_path="$1"
	local website_inc_config="${prev_pkg_path}/config.inc.php"
	[ ! -s "$website_inc_config" ];
}

CustomHasRunWebsiteSetup()
{
	local prev_pkg_path="$1"
	# this behavior refers to isInstalled() in vtigercrm/includes/main/WebUI.php
	if IsIncConfigEmpty $prev_pkg_path ; then
		false
		return
	fi
	true
}

CustomGetBackupPkgPath()
{
	local backup_prefix_path=$(CustomGetBackupPrefix)
	local backup_pkg_path=""
	local pkg_dir_540="vtigercrm-5.4.0"

	backup_pkg_path="$backup_prefix_path/$PKG_DIR"
	if [ ! -d "$backup_pkg_path" ]; then
		backup_pkg_path="$backup_prefix_path/$pkg_dir_540"
	fi
	echo "$backup_pkg_path"
}

CustomParseDBConf()
{
	local path="$1"
	local db_info="$2"
	local grep_info="$3"
	local info_output=$(CustomGetValueFromMetaFile "$db_info")

	if [ -z "$info_output" ]; then
		if [ -s "$path/$CONF_FILE" ]; then
			info_output=$(grep "dbconfig\['$grep_info'\]" "$path/$CONF_FILE" | cut -d"'" -f4)
		elif [ -s "$path/$DB_SETTING" ]; then
			info_output=$(grep "$grep_info' =>" "$path/$DB_SETTING" | cut -d"'" -f4)
		fi
	fi
	echo "$info_output"

}

CustomGetDBName()
{
	CustomParseDBConf "$1" "db_name" "db_name"
}

CustomGetDBUser()
{
	CustomParseDBConf "$1" "db_user" "db_username"
}

CustomGetDBPass()
{
	CustomParseDBConf "$1" "db_pass" "db_password"
}
wizard_db_settings="Configurare il database vtigerCRM"
wizard_db_name_desc="Immettere il nome database per il database vtigerCRM."
wizard_db_name_label="Nome database"
wizard_m10_desc="Inserire le credenziali amministratore di MariaDB 10 per procedere con le impostazioni del database vtigerCRM."
wizard_admin_acc="Account"
wizard_admin_pass="Password"
wizard_set_db_desc="Creare un nuovo account database esclusivo per vtigerCRM per usare i dati nuovi/esistenti."
wizard_set_data_title="Configurare vtigerCRM"
wizard_found_backup="Dati riservati al pacchetto incompatibili"
wizard_warn_data_backup_deprecated="I dati riservati di questo pacchetto nel sistema non sono compatibili con questa versione.<br>Se si installa questa versione, i dati riservati saranno cancellati.<br>Per mantenere i dati riservati, installare il pacchetto della stessa versione della precedente installazione. (Versione precedente: {prev_version})"
wizard_db_user_account_desc="Account utente database"
wizard_db_user_password_desc="Password utente database"
wizard_migrate_note="L'ultima versione di vtigerCRM supporta solo MariaDB 10.<br>Il sistema ha rilevato un database MariaDB 5 esistente.<br>Nel prossimo passaggio, il database sarà migrato da MariaDB 5 a MariaDB 10.<br><br>Inserire le credenziali amministratore di MariaDB 5 per procedere con l'installazione."
wizard_migrate_title="Migrare il database vtigerCRM"
wizard_remove_msql_title="Rimuovi database vtigerCRM"
wizard_remove_msql_desc="Se il database vtigerCRM viene rimosso, tutti i dati saranno eliminati."
wizard_msql_password_desc_remove="Inserire le credenziali dell'amministratore del database per rimuovere il database vtigerCRM."
wizard_admin_acc="Account"
wizard_admin_pass="Password"
wizard_unable_to_upgrade_title="Impossibile aggiornare a questa versione"
wizard_upgrade_from_prev_release_required_desc="Prima di aggiornare il pacchetto a questa versione, aggiornarlo alla <a href=\"{1}\" target=\"_blank\">versione precedente</a>."
PageUninst()
{
cat << EOF
{
	"step_title": "$wizard_remove_msql_title",
	"items": [{
		"type": "multiselect",
		"subitems": [{
			"key": "pkgwizard_remove_mysql",
			"defaultValue": true,
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"desc": "$wizard_msql_password_desc_remove",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root"
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
main()
{
	local uninst_page="$(PageUninst)"
	echo "[$uninst_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
